@@ -460,6 +460,23 @@ calendar event."
           (('command command)
            (format #t (l10n "  Command: ~a~%")
                    (shell-quoted-command command)))))
+       (('system-log ('version 0)
+                     ('endpoints (('endpoint ('version 0)
+                                             ('name names)
+                                             ('address addresses)
+                                             _ ...) ...))
+                     ('kernel-log-file kernel-log-file)
+                     rest ...)
+        (format #t (l10n "  System log listening on ~a endpoint:~%"
+                         "  System log listening on ~a endpoints:~%"
+                         (length names))
+                (length names))
+        (for-each (lambda (name address)
+                    (format #t "    - ~a (~s)~%"
+                            (socket-address->string address) name))
+                  names addresses)
+        (format #t "  Kernel log file: ~a~%"
+                (or kernel-log-file "-")))
        (('timer ('version 0)
                 ('event event) ('action action)
                 ('processes processes) _ ...)
