@@ -26,7 +26,8 @@
                                   open-log-file
                                   log-line
                                   rotate-and-reopen-log-file)
-  #:autoload   (shepherd comm) (system-log-file)
+  #:autoload   (shepherd comm) (system-log-file
+                                define-record-type-serializer)
   #:use-module (srfi srfi-1)
   #:use-module (srfi srfi-9)
   #:use-module (srfi srfi-9 gnu)
@@ -163,8 +164,12 @@ or @code{#f}) as its sender."
 will be printed."
   (string-append (or (and=> (system-log-message-sender message)
                             (lambda (address)
-                              (string-append (socket-address->string address)
-                                             " ")))
+                              ;; On the Hurd, accept(2) returns a socket
+                              ;; address with the empty string for incoming
+                              ;; AF_UNIX connections.  Ignore it.
+                              (and (not (= AF_UNIX (sockaddr:fam address)))
+                                   (string-append (socket-address->string address)
+                                                  " "))))
                      "localhost ")
                  (if (= (system-log-message-facility message)
                         (system-log-facility kernel))
@@ -229,18 +234,23 @@ and passing them to @var{dispatcher}."
         (and (input-port? obj)
              (not (memq obj non-socket))))))
 
-  (let loop ()
+  (let loop ((ports ports))
     (match (wait-for-input-or-message ports channel)
       ((? socket-port? socket)                    ;socket such as /dev/log
        (match (recvfrom! socket buffer)
          ((0 . _) #t)
          ((bytes . sender) (log-bytes buffer bytes sender)))
-       (loop))
+       (loop ports))
       ((? input-port? port)                      ;regular file like /proc/kmsg
        (match (read-line port)
-         ((? eof-object?) #t)
-         (line (put-message dispatcher (parse-system-log-message line))))
-       (loop))
+         ((? eof-object?)
+          (local-output (l10n "Closing system log input file ~s on end-of-file.")
+                        (or (port-filename port) (fileno port)))
+          (close-port port)
+          (loop (delq port ports)))
+         (line
+          (put-message dispatcher (parse-system-log-message line))
+          (loop ports))))
       (('terminate reply)
        ;; Close all of PORTS.  DISPATCHER itself is registered as a logger and
        ;; thus shut down separately, by the service itself.
@@ -392,9 +402,17 @@ default destination to log it to."
                 '())))
       (const (list (in-vicinity %user-log-dir "syslog")))))
 
+(define %default-kernel-log-file
+  (cond ((string-contains %host-type "linux")     ;Linux
+         "/proc/kmsg")
+        ((string-suffix? "-gnu" %host-type)
+         "/dev/klog")                             ;GNU/Hurd
+        (else
+         #f)))
+
 (define kernel-log-file
   ;; File to read to get kernel messages.
-  (make-parameter "/proc/kmsg"))
+  (make-parameter %default-kernel-log-file))
 
 (define default-max-silent-time
   ;; Maximum number of seconds during which syslogd should remain silent.
@@ -402,11 +420,15 @@ default destination to log it to."
 
 ;; Running system log.
 (define-record-type <system-log>
-  (system-log channel ports dispatcher)
+  (system-log channel ports dispatcher endpoints kernel-log-file)
   system-log?
   (channel    system-log-channel)
   (ports      system-log-ports)
-  (dispatcher system-log-dispatcher))
+  (dispatcher system-log-dispatcher)
+
+  ;; The following fields are informational.
+  (endpoints  system-log-endpoints)               ;list of <endpoint>
+  (kernel-log-file system-log-kernel-log-file))   ;#f | file name
 
 (define (print-system-log log port)
   (simple-format port "#<system-log ~a>"
@@ -414,6 +436,17 @@ default destination to log it to."
 
 (set-record-type-printer! <system-log> print-system-log)
 
+(define-record-type-serializer (system-log->sexp (system-log <system-log>))
+  `(system-log (version 0)
+               (endpoints
+                ,(map endpoint->sexp (system-log-endpoints system-log)))
+               (kernel-log-file ,(system-log-kernel-log-file system-log))
+               (sockets ,(filter-map (lambda (port)
+                                       (and (not (port-closed? port))
+                                            (file-port? port)
+                                            (fileno port)))
+                                     (system-log-ports system-log)))))
+
 (define (file->endpoint file)
   "Return a endpoint for Unix-domain socket @var{file}."
   (endpoint (make-socket-address AF_UNIX file)
@@ -437,7 +470,7 @@ default destination to log it to."
 @var{provision} and @var{requirement} (lists of symbols).  The service accepts
 connections on @var{sources}, a list of @code{<endpoint>} objects; optionally
 it also reads messages from @code{#:kernel-log-file}, which defaults to
-@file{/proc/kmsg} when running as root.
+@file{/proc/kmsg} (Linux) or @file{/dev/klog} (the Hurd) when running as root.
 
 Log messages are passed to @var{message-destination}, a one-argument procedure
 that must return the list of files to write it to.  Write a mark to log files
@@ -448,30 +481,43 @@ for a format identical to that of traditional syslogd implementations.
 
 Keep up to @var{history-size} messages in memory for the purposes of allowing
 users to view recent messages without opening various files."
+  (define (maybe-kernel-log-port)
+    (catch 'system-error
+      (lambda ()
+        (open kernel-log-file
+              (logior O_RDONLY O_NONBLOCK O_CLOEXEC)))
+      (lambda args
+        ;; If KERNEL-LOG-FILE cannot be opened, dismiss it instead of failing
+        ;; to start since that could prevent many system services from
+        ;; starting (useful for example in Linux containers where /proc/kmsg
+        ;; is not accessible).
+        (local-output (l10n "Dismissing kernel log '~a': ~a.")
+                      kernel-log-file
+                      (strerror (system-error-errno args)))
+        #f)))
+
   (define this-system-log
     (service provision
              #:requirement requirement
              #:start (lambda ()
-                       (let ((channel (make-channel))
-                             (ports (append (open-sockets sources)
-                                            (if kernel-log-file
-                                                (list (open kernel-log-file
-                                                            (logior O_RDONLY
-                                                                    O_NONBLOCK
-                                                                    O_CLOEXEC)))
-                                                '())))
-                             (dispatcher (spawn-log-dispatcher message-destination
-                                                               #:max-silent-time
-                                                               max-silent-time
-                                                               #:history-size
-                                                               history-size
-                                                               #:date-format
-                                                               date-format)))
+                       (let* ((channel (make-channel))
+                              (klog (and kernel-log-file
+                                         (maybe-kernel-log-port)))
+                              (ports (append (open-sockets sources)
+                                             (if klog (list klog) '())))
+                              (dispatcher (spawn-log-dispatcher message-destination
+                                                                #:max-silent-time
+                                                                max-silent-time
+                                                                #:history-size
+                                                                history-size
+                                                                #:date-format
+                                                                date-format)))
                          (register-service-logger this-system-log dispatcher)
                          (spawn-fiber
                           (lambda ()
                             (run-system-log channel ports dispatcher)))
-                         (system-log channel ports dispatcher)))
+                         (system-log channel ports dispatcher
+                                     sources (and klog kernel-log-file))))
              #:stop (lambda (system-log)
                       (let ((reply (make-channel)))
                         (put-message (system-log-channel system-log)
