@@ -1,5 +1,5 @@
 # GNU Shepherd --- Test basic communication capabilities.
-# Copyright © 2013-2014, 2016-2019, 2022-2024 Ludovic Courtès <ludo@gnu.org>
+# Copyright © 2013-2014, 2016-2019, 2022-2025 Ludovic Courtès <ludo@gnu.org>
 # Copyright © 2016 Mathieu Lirzin <mthl@gnu.org>
 # Copyright © 2014 Alex Sassmannshausen <alex.sassmannshausen@gmail.com>
 #
@@ -74,6 +74,8 @@ cat > "$conf"<<EOF
 	 #:respawn? #f)))
 EOF
 
+shepherd -I -s /does/not/exist/sock -c "$conf" && false
+
 rm -f "$pid"
 shepherd -I -s "$socket" -c "$conf" -l "$log" --pid="$pid" &
 
@@ -84,6 +86,10 @@ shepherd_pid="`cat $pid`"
 
 kill -0 $shepherd_pid
 test -S "$socket"
+
+# Attempt to reuse $socket should fail.
+shepherd -I -s "$socket" -c "$conf" && false
+
 pristine_status=`$herd status -n 0` # Prep for 'reload' test.
 echo $pristine_status | grep -E '(Start.*root|Stop.*test)'
 
