@@ -1,5 +1,5 @@
 # GNU Shepherd --- Test the #:pid-file option of 'make-forkexec-constructor'.
-# Copyright © 2016, 2019, 2020, 2022, 2023 Ludovic Courtès <ludo@gnu.org>
+# Copyright © 2016, 2019-2020, 2022-2025 Ludovic Courtès <ludo@gnu.org>
 #
 # This file is part of the GNU Shepherd.
 #
@@ -59,6 +59,12 @@ cat > "$conf"<<EOF
                             (display (getpid) port)))
                         (sleep 100))))))
 
+(define nonexistent-pid-file
+  ;; Nonexistent PID file in a writable directory, so unlink(2) fails
+  ;; with ENOENT rather than EROFS as is the case in a container with
+  ;; a read-only root file system.
+  "$PWD/$$/does-not-exist")
+
 (register-services
  (list
   (service
@@ -66,7 +72,7 @@ cat > "$conf"<<EOF
     ;; behind it.
     '(test)
     #:start (make-forkexec-constructor %command
-				       #:pid-file "/does-not-exist"
+				       #:pid-file nonexistent-pid-file
 
 				       ;; Low-end ARMv7 machines are
 				       ;; slow enough that creating
@@ -90,7 +96,7 @@ cat > "$conf"<<EOF
     ;; a child process behind it.
     '(test-daemonizes)
     #:start (make-forkexec-constructor %daemon-command
-				       #:pid-file "/does-not-exist"
+				       #:pid-file nonexistent-pid-file
 				       #:pid-file-timeout 6)
     #:stop  (make-kill-destructor)
     #:respawn? #f)))
@@ -151,8 +157,8 @@ $herd start test-works
 $herd status test-works | grep running
 test -f "$service_pid"
 kill -0 "`cat $service_pid`"
-known_pid="`$herd status test-works | grep Running \
-   | sed -es'/.*Running value.* \([0-9]\+\)\.$/\1/g'`"
+known_pid="`$herd status test-works | grep PID \
+   | sed -es'/.*PID: \([0-9]\+\)$/\1/g'`"
 test `cat $service_pid` -eq $known_pid
 
 $herd stop test-works
