@@ -1,5 +1,5 @@
 # GNU Shepherd --- Test the "starting" status.
-# Copyright © 2022, 2023 Ludovic Courtès <ludo@gnu.org>
+# Copyright © 2022-2023, 2025 Ludovic Courtès <ludo@gnu.org>
 #
 # This file is part of the GNU Shepherd.
 #
@@ -128,9 +128,9 @@ done
 grep "Waiting for test to start" "$log"
 
 touch "$stamp"			# trigger starting->running transition
-while kill -0 $herd_start_pid; do sleep 0.5; done
-while kill -0 $herd_stop_pid1; do sleep 0.5; done
-while kill -0 $herd_stop_pid2; do sleep 0.5; done
+wait $herd_start_pid
+wait $herd_stop_pid1
+wait $herd_stop_pid2
 $herd status test | grep stopped
 
 
