@@ -1,5 +1,5 @@
 # GNU Shepherd --- Test whether 'system*' is blocking.
-# Copyright © 2022-2023 Ludovic Courtès <ludo@gnu.org>
+# Copyright © 2022-2023, 2025 Ludovic Courtès <ludo@gnu.org>
 #
 # This file is part of the GNU Shepherd.
 #
@@ -33,6 +33,8 @@ trap "cat $log || true; rm -f $socket $conf $stamp $log;
 script="while [ ! -f $PWD/$stamp ] ; do sleep 0.1 ; done ; exit \$(cat $PWD/$stamp)"
 
 cat > "$conf" <<EOF
+(use-modules ((shepherd support) #:select (with-directory-excursion)))
+
 (register-services
  (list (service
 	 '(test)
@@ -54,6 +56,12 @@ cat > "$conf" <<EOF
 		   (zero? (system* "this command does not exist")))
 	 #:stop  (const #f)
 	 #:respawn? #f)
+       (service
+	 '(test-current-directory)
+	 #:start (lambda _
+		   (with-directory-excursion "${TMPDIR:-/tmp}"
+                     (zero? (system "echo CWD: \$PWD"))))
+	 #:respawn? #f)
        (service
 	 '(test-with-respawn)
 	 #:start (make-forkexec-constructor
@@ -122,6 +130,13 @@ $herd start test-command-not-found && false
 $herd status test-command-not-found
 $herd status test-command-not-found | grep "stopped"
 
+# Check that 'system*' and 'system' run commands from the current working
+# directory rather than from (default-service-directory).
+$herd start test-current-directory
+$herd status test-current-directory | grep running
+grep "CWD: ${TMPDIR:-/tmp}" "$log"
+$herd stop test-current-directory
+
 # What about a service with a custom 'stop' procedure that uses 'system*'?
 # Stopping the service should not trigger the respawn machinery.
 $herd start test-with-respawn
